@@ -1,13 +1,16 @@
 #
-# $Id: MediaTypes.pm,v 1.14 1996/05/08 16:28:55 aas Exp $
+# $Id: MediaTypes.pm,v 1.19 1997/12/02 13:22:52 aas Exp $
 
 package LWP::MediaTypes;
 
 =head1 NAME
 
-guess_media_type - guess media type for a file or a URL.
+LWP::MediaTypes - guess media type for a file or a URL.
 
-media_suffix - returns file extentions for a media type
+=head1 SYNOPSIS
+
+ use LWP::MediaTypes qw(guess_media_type);
+ $type = guess_media_type("/tmp/foo.gif");
 
 =head1 DESCRIPTION
 
@@ -15,9 +18,12 @@ This module provides functions for handling of media (also known as
 MIME) types and encodings.  The mapping from file extentions to media
 types is defined by the F<media.types> file.  If the F<~/.media.types>
 file exist it is used as a replacement.
-
 For backwards compatability we will also look for F<~/.mime.types>.
 
+The following functions are available (and exported by default):
+
+=over 4
+
 =cut
 
 ####################################################################
@@ -27,14 +33,26 @@ require Exporter;
 @EXPORT = qw(guess_media_type media_suffix);
 
 require LWP::Debug;
+use strict;
 
-my %suffixType = (              # note: initialized from mime.types
+# note: These hashes will also be filled with the entries found in
+# the 'media.types' file.
+
+my %suffixType = (
     'txt'   => 'text/plain',
     'html'  => 'text/html',
     'gif'   => 'image/gif',
     'jpg'   => 'image/jpeg',
 );
 
+my %suffixExt = (
+    'text/plain' => 'txt',
+    'text/html'  => 'h',
+    'image/gif'  => 'gif',
+    'image/jpeg' => 'jpg',
+);
+
+#XXX: there should be some way to define this in the media.types files.
 my %suffixEncoding = (
     'Z'   => 'compress',
     'gz'  => 'gzip',
@@ -45,11 +63,14 @@ my %suffixEncoding = (
 
 local($/, $_) = ("\n", undef);  # ensure correct $INPUT_RECORD_SEPARATOR
 
+my @priv_files = ();
+push(@priv_files, "$ENV{HOME}/.media.types", "$ENV{HOME}/.mime.types")
+  if defined $ENV{HOME};  # Some does not have a home (for instance Win32)
+
 # Try to locate "media.types" file, and initialize %suffixType from it
-for $typefile ((map {"$_/LWP/media.types"} @INC),
-	       "$ENV{HOME}/.media.types",
-	       "$ENV{HOME}/.mime.types",
-	       ) {
+my $typefile;
+for $typefile ((map {"$_/LWP/media.types"} @INC), @priv_files) {
+    local(*TYPE);
     open(TYPE, $typefile) || next;
     LWP::Debug::debug("Reading media types from $typefile");
     while (<TYPE>) {
@@ -57,6 +78,8 @@ for $typefile ((map {"$_/LWP/media.types"} @INC),
 	next if /^\s*$/; # blank line
 	s/#.*//;         # remove end-of-line comments
 	my($type, @exts) = split(' ', $_);
+	$suffixExt{$type} = $exts[0] if @exts;
+	my $ext;
 	for $ext (@exts) {
 	    $suffixType{$ext} = $type;
 	}
@@ -67,9 +90,7 @@ for $typefile ((map {"$_/LWP/media.types"} @INC),
 
 ####################################################################
 
-=head1 FUNCTIONS
-
-=head2 guess_media_type($filename)
+=item guess_media_type($filename_or_url, [$header_to_modify])
 
 This function tries to guess media type and encoding for given file.
 In scalar context it returns only the content-type.  In array context
@@ -84,11 +105,16 @@ C<-T> perl operator in order to determine if this is a text file
 (text/plain).  If this does not work it will return
 I<application/octet-stream> as the type.
 
+The optional second argument should be a reference to a HTTP::Headers
+object (or some HTTP::Message object).  When present this function
+will set the value of the 'Content-Type' and 'Content-Encoding' for
+this header.
+
 =cut
 
 sub guess_media_type
 {
-    my($file) = @_;
+    my($file, $header) = @_;
     return undef unless defined $file;
 
     my $fullname;
@@ -138,29 +164,57 @@ sub guess_media_type
 	}
     }
 
+    if ($header) {
+	$header->header('Content-Type' => $ct);
+	$header->header('Content-Encoding' => \@encoding) if @encoding;
+    }
+
     wantarray ? ($ct, @encoding) : $ct;
 }
 
 
-=head2 media_suffix($type)
-
-  media_suffix('image/*')
+=item media_suffix($type,...)
 
 This function will return all suffixes that can be used to denote the
-specified media type.  Wildcard types can be used.
+specified media type(s).  Wildcard types can be used.  In scalar
+context it will return the first suffix found.
+
+Examples:
+
+  @suffixes = media_suffix('image/*', 'audio/basic');
+  $suffix = media_suffix('text/html');
 
 =cut
 
 sub media_suffix {
+    if (!wantarray && @_ == 1 && $_[0] !~ /\*/) {
+	return $suffixExt{$_[0]};
+    }
     my(@type) = @_;
-    my(@suffix,$nom,$val);
+    my(@suffix, $ext, $type);
     foreach (@type) {
-	s/\*/.*/;
-	while(($nom,$val) = each(%suffixType)) {
-	    push(@suffix, $nom) if $val =~ /^$_$/;
+	if (s/\*/.*/) {
+	    while(($ext,$type) = each(%suffixType)) {
+		push(@suffix, $ext) if $type =~ /^$_$/;
+	    }
+	} else {
+	    while(($ext,$type) = each(%suffixType)) {
+		push(@suffix, $ext) if $type eq $_;
+	    }
 	}
     }
-    @suffix;
+    wantarray ? @suffix : $suffix[0];
 }
 
 1;
+
+=back 
+
+=head1 COPYRIGHT
+
+Copyright 1995-1997 Gisle Aas.
+
+This library is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
+=cut
