@@ -1,4 +1,4 @@
-# $Id: RobotUA.pm,v 1.4 1996/04/09 15:44:29 aas Exp $
+# $Id: RobotUA.pm,v 1.12 1997/12/15 19:54:56 aas Exp $
 
 package LWP::RobotUA;
 
@@ -13,6 +13,7 @@ use Carp ();
 use LWP::Debug ();
 use HTTP::Status ();
 use HTTP::Date qw(time2str);
+use strict;
 
 =head1 NAME
 
@@ -31,29 +32,23 @@ LWP::RobotUA - A class for Web Robots
 
 This class implements a user agent that is suitable for robot
 applications.  Robots should be nice to the servers they visit.  They
-should consult the /robots.txt file to ensure that they are welcomed
+should consult the F<robots.txt> file to ensure that they are welcomed
 and they should not send too frequent requests.
 
 But, before you consider writing a robot take a look at
 <URL:http://info.webcrawler.com/mak/projects/robots/robots.html>.
 
-When you use a LWP::RobotUA as your user agent, then you do not really
-have to think about these things yourself.  Just send requests as you
-do when you are using a normal LWP::UserAgent and this special agent
-will make sure you are nice.
-
-=head1 SEE ALSO
-
-L<LWP::UserAgent>
+When you use a I<LWP::RobotUA> as your user agent, then you do not
+really have to think about these things yourself.  Just send requests
+as you do when you are using a normal I<LWP::UserAgent> and this
+special agent will make sure you are nice.
 
 =head1 METHODS
 
 The LWP::RobotUA is a sub-class of LWP::UserAgent and implements the
-same methods.  The use_alarm() method also desides whether we will
-wait if a request is tried too early (if true), or will return an error
-response (if false).
+same methods. In addition the following methods are provided:
 
-In addition these methods are provided:
+=over 4
 
 =cut
 
@@ -64,28 +59,23 @@ In addition these methods are provided:
 # $self->{'delay'}    Required delay between request to the same
 #                     server in minutes.
 #
-# $self->{'visited'}   An hash where the keys are server names and the
-#                      value is a hash with these values:
-#                          'last'   last fetch time
-#                          'count'  number of documents fetched
-#
 # $self->{'rules'}     A WWW::RobotRules object
 #
 
-=head2 $ua = LWP::RobotUA->new($agent_name, $from)
 
-A name and the mail address of the human running the the robot is
-required by the constructor.  The name can be changed later though the
-agent() method.  The mail address chan be changed with the from()
-method.
+=item $ua = LWP::RobotUA->new($agent_name, $from, [$rules])
+
+Your robot's name and the mail address of the human responsible for
+the robot (i.e. you) is required by the constructor.
+
+Optionally it allows you to specify the I<WWW::RobotRules> object to
+use.
 
 =cut
 
 sub new
 {
-    my $class = shift;
-    my $name  = shift;
-    my $from  = shift;
+    my($class,$name,$from,$rules) = @_;
 
     Carp::croak('LWP::RobotUA name required') unless $name;
     Carp::croak('LWP::RobotUA from address required') unless $from;
@@ -96,21 +86,36 @@ sub new
     $self->{'delay'} = 1;   # minutes
     $self->{'agent'} = $name;
     $self->{'from'}  = $from;
+    $self->{'use_sleep'} = 1;
 
-    $self->{'rules'} = new WWW::RobotRules $name;
-    $self->{'visited'} = { };
+    if ($rules) {
+	$rules->agent($name);
+	$self->{'rules'} = $rules;
+    } else {
+	$self->{'rules'} = new WWW::RobotRules $name;
+    }
 
     $self;
 }
 
-=head2 $ua->delay([$minutes])
+
+=item $ua->delay([$minutes])
 
 Set the minimum delay between requests to the same server.  The
 default is 1 minute.
 
+=item $ua->use_sleep([$boolean])
+
+Get/set a value indicating wether the UA should sleep() if request
+arrive to fast (before $ua->delay minutes has passed).  The default is
+TRUE.  If this value is FALSE then an internal SERVICE_UNAVAILABLE
+response will be generated.  It will have an Retry-After header that
+indicate when it is OK to send another request to this server.
+
 =cut
 
-sub delay { shift->_elem('delay', @_); }
+sub delay     { shift->_elem('delay',     @_); }
+sub use_sleep { shift->_elem('use_sleep', @_); }
 
 sub agent
 {
@@ -118,89 +123,113 @@ sub agent
     my $old = $self->SUPER::agent(@_);
     if (@_) {
 	# Changing our name means to start fresh
-	$self->{'rules'} = new WWW::RobotRules $self->{'agent'};
-	$self->{'visited'} = {};
+	$self->{'rules'}->agent($self->{'agent'}); 
     }
     $old;
 }
 
-=head2 $ua->host_count($hostname)
 
-Returns the number of documents fetched from this server host.
+=item $ua->rules([$rules])
+
+Set/get which I<WWW::RobotRules> object to use. 
 
 =cut
 
-sub host_count
-{
-    my($self, $host) = @_;
-    return undef unless defined $host;
-    if ($self->{'visited'}{$host}) {
-	return $self->{'visited'}{$host}{'count'};
-    }
-    return undef;
+sub rules {
+    my $self = shift;
+    my $old = $self->_elem('rules', @_);
+    $self->{'rules'}->agent($self->{'agent'}) if @_;
+    $old;
 }
 
-=head2 $ua->host_wait($hostname)
 
-Returns the number of seconds you must wait before you can make a new
-request to this host.
+=item $ua->no_visits($netloc)
+
+Returns the number of documents fetched from this server host. Yes I
+know, this method should probably have been named num_visits() or
+something like that :-(
+
+=cut
+
+sub no_visits
+{
+    my($self, $netloc) = @_;
+    $self->{'rules'}->no_visits($netloc);
+}
+
+*host_count = \&no_visits;  # backwards compatibility with LWP-5.02
+
+
+=item $ua->host_wait($netloc)
+
+Returns the number of seconds (from now) you must wait before you can
+make a new request to this host.
 
 =cut
 
 sub host_wait
 {
-    my($self, $host) = @_;
-    return undef unless defined $host;
-    if ($self->{'visited'}{$host}) {
-	my $wait = int($self->{'delay'} * 60 -
-		       (time - $self->{'visited'}{$host}{'last'}));
+    my($self, $netloc) = @_;
+    return undef unless defined $netloc;
+    my $last = $self->{'rules'}->last_visit($netloc);
+    if ($last) {
+	my $wait = int($self->{'delay'} * 60 - (time - $last));
 	$wait = 0 if $wait < 0;
 	return $wait;
     }
     return 0;
 }
 
+
 sub simple_request
 {
     my($self, $request, $arg, $size) = @_;
 
     LWP::Debug::trace('()');
 
-    my $host = $request->url->host;
-
     # Do we try to access a new server?
-    unless ($self->{'visited'}{$host}) {
-	LWP::Debug::debug("Host $host is not visited before.");
-	$self->{'visited'}{$host}{'count'} = 0;  # avoids infinite recursion
-	$self->{'visited'}{$host}{'last'}  = 0;
+    my $allowed = $self->{'rules'}->allowed($request->url);
+
+    if ($allowed < 0) {
+	LWP::Debug::debug("Host is not visited before, or robots.txt expired.");
 	# fetch "robots.txt"
 	my $robot_url = $request->url->clone;
 	$robot_url->path("robots.txt");
 	$robot_url->params(undef);
 	$robot_url->query(undef);
 	LWP::Debug::debug("Requesting $robot_url");
+
+	# make access to robot.txt legal since this will be a recursive call
+	$self->{'rules'}->parse($robot_url, ""); 
+
 	my $robot_req = new HTTP::Request 'GET', $robot_url;
-	# This will be a recursive call
 	my $robot_res = $self->request($robot_req);
+	my $fresh_until = $robot_res->fresh_until;
 	if ($robot_res->is_success) {
 	    LWP::Debug::debug("Parsing robot rules");
-	    $self->{'rules'}->parse($robot_url, $robot_res->content);
+	    $self->{'rules'}->parse($robot_url, $robot_res->content, 
+				    $fresh_until);
 	} else {
-	    LWP::Debug::debug("No robots.txt file for $host");
+	    LWP::Debug::debug("No robots.txt file found");
+	    $self->{'rules'}->parse($robot_url, "", $fresh_until);
 	}
+
+	# recalculate allowed...
+	$allowed = $self->{'rules'}->allowed($request->url);
     }
 
     # Check rules
-    LWP::Debug::debug("Checking robot rules");
-    unless ($self->{'rules'}->allowed($request->url)) {
+    unless ($allowed) {
 	return new HTTP::Response
 	  &HTTP::Status::RC_FORBIDDEN, 'Forbidden by robots.txt';
     }
-    my $wait = $self->host_wait($host);
+
+    my $netloc = $request->url->netloc;
+    my $wait = $self->host_wait($netloc);
 
     if ($wait) {
 	LWP::Debug::debug("Must wait $wait seconds");
-	if ($self->{'use_alarm'}) {
+	if ($self->{'use_sleep'}) {
 	    sleep($wait)
 	} else {
 	    my $res = new HTTP::Response
@@ -213,13 +242,13 @@ sub simple_request
     # Perform the request
     my $res = $self->SUPER::simple_request($request, $arg, $size);
 
-    $self->{'visited'}{$host}{'last'} = time;
-    $self->{'visited'}{$host}{'count'}++;
+    $self->{'rules'}->visit($netloc);
 
     $res;
 }
 
-=head2 $ua->as_string
+
+=item $ua->as_string
 
 Returns a text that describe the state of the UA.
 Mainly useful for debugging.
@@ -232,23 +261,24 @@ sub as_string
     my @s;
     push(@s, "Robot: $self->{'agent'} operated by $self->{'from'}  [$self]");
     push(@s, "    Minimum delay: " . int($self->{'delay'}*60) . "s");
-    push(@s, "    Will sleep if too early") if $self->{'use_alarm'};
+    push(@s, "    Will sleep if too early") if $self->{'use_sleep'};
     push(@s, "    Rules = $self->{'rules'}");
-    push(@s, "    Visits");
-    for (sort keys %{$self->{'visited'}}) {
-	my $e = $self->{visited}{$_};
-	push(@s, sprintf "      %-20s: " .
-			 localtime($e->{'last'}) . "%4d",
-			 $_, $e->{'count'});
-    }
-
     join("\n", @s, '');
 }
 
 1;
 
-=head1 AUTHOR
+=back
 
-Gisle Aas <aas@sn.no>
+=head1 SEE ALSO
+
+L<LWP::UserAgent>, L<WWW::RobotRules>
+
+=head1 COPYRIGHT
+
+Copyright 1996-1997 Gisle Aas.
+
+This library is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
 
 =cut
